/*
 * $Header: /home/gene/src/lisp-script/src/RCS/io.c,v 1.3 2004/03/25 09:56:50 gene Exp $
 */

/*
 * Copyright (c) 2004.  Gene Michael Stover.  All rights reserved.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "this.h"

int
IO_CopyStreams (dst, src)
     FILE *dst;
     FILE *src;
{
  int rc = 0;
  int ch;

  while (rc == 0 && (ch = getc (src)) != -1) {
    if (fputc (ch, dst) == ch) {
      /* good */
    } else {
      fprintf (stderr, "\nfputc: %s", strerror (errno));
      fprintf (stderr, "\n%s:%d: fputc failed", __FILE__, __LINE__);
      rc = 3;
    }
  }
  return rc;
}

void
IO_DiscardLine (fp)
     FILE *fp;
{
  int ch;

  do {
    ch = fgetc (fp);
  } while (ch != '\n' && ch != -1);
}

char *
IO_GetTmpDir ()
{
  static char tmp[] = "/tmp";

  if (getenv ("TMPDIR")) {
    return getenv ("TMPDIR");
  } else if (IO_IsDirectory (tmp) && access (tmp, R_OK | W_OK | X_OK) == 0) {
    return tmp;
  } else {
    return "./";
  }
}

Boolean
IO_IsDirectory (pathname)
     char pathname[];
{
  struct stat ss;

  return stat (pathname, &ss) == 0 && S_ISDIR (ss.st_mode);
}

char *
IO_MakeUniqueFile ()
{
  char *pathname;
  int fd = -1;

  do {
    pathname = IO_TmpPathname ();
    fd = open (pathname, O_WRONLY | O_CREAT | O_EXCL, 0600);
    if (fd == -1) {
      xfree (pathname);
    }
  } while (fd == -1);
  close (fd);
  return pathname;
}

char *
IO_TmpPathname ()
{
  char *pathname;
  int sz;

  /* The size we'l need is the length of the tmpdir string, plus
   * the maximum width (in characters) of a random number (times
   * two because we use two random numbers), plus a couple of
   * characters for the / and -, plus one for the null-terminator. */
  sz = strlen (IO_GetTmpDir ()) +       /* tmpdir length */
               2 * log10 (INT_MAX) + 2 + /* length of two random numbers */
               2 +                      /* for the / and - chars */
               1;                       /* terminator */
  pathname = (char *) xmalloc (sz);
  sprintf (pathname, "%s/%X-%X", IO_GetTmpDir (), rand (), rand ());
  assert (strlen (pathname) < sz);
  return pathname;
}

/* --- end of file --- */
