/*
 * $Header: /home/gene/src/lisp-script/src/RCS/lisp-script.c,v 1.7 2004/04/06 07:45:15 gene Exp $
 */

/*
 * Copyright (c) 2004.  Gene Michael Stover.  All rights reserved.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "this.h"

static char *S_script_pathname = NULL;

static int
S_CommandLine (argc, argv)
     int argc;
     char *argv[];
{
  int rc = 0;

  if (argc >= 2) {
    S_script_pathname = argv[1];
    if (argc >= 3) {
      fprintf (stderr, "\n%s:%d:", __FILE__, __LINE__);
      fprintf (stderr, " command line arguments are not yet supported");
      rc = 17;
    }
  } else {
    fprintf (stderr, "\n%s:%d:", __FILE__, __LINE__);
    fprintf (stderr, " script file name is missing from the command line");
    rc = 4;
  }
  return rc;
}

static int
S_Init ()
{
  int rc = 0;
  
  srand (time (NULL) + getpid ());
  return rc;
}

static char *
S_MakeTempfile (script)
     char script[];
{
  char *pathname = NULL;
  FILE *script_fp, *tmp_fp;

  script_fp = fopen (script, "r");
  if (script_fp != NULL) {
    pathname = IO_MakeUniqueFile ();
    tmp_fp = fopen (pathname, "w");
    if (tmp_fp != NULL) {
      LISP_WriteBeginning (tmp_fp);
      IO_DiscardLine (script_fp);
      IO_CopyStreams (tmp_fp, script_fp);
      LISP_WriteEnding (tmp_fp);
      fclose (tmp_fp);
    } else {
      fprintf (stderr, "\nfopen: %s", strerror (errno));
      fprintf (stderr, "\n%s:%d: fopen failed", __FILE__, __LINE__);
      xfree (pathname);
      pathname = NULL;
    }
    fclose (script_fp);
  } else {
    fprintf (stderr, "\nfopen: %s", strerror (errno));
    fprintf (stderr, "\n%s:%d: can't open script file \"%s\"", __FILE__,
	     __LINE__, script);
    assert (pathname == NULL);
  }
  return pathname;
}

static int S_saved_stderr = -1;

static void
S_PrintError (pathname)
     char pathname[];
{
  FILE *fp;

  fprintf (stderr, "\n%s:%d: dump of Lisp interpreter's error output:",
	   __FILE__, __LINE__);
  fp = fopen (pathname, "r");
  if (fp != NULL) {
    if (IO_CopyStreams (stderr, fp) == 0) {
      /* good */
    } else {
      fprintf (stderr, "\n%s:%d: IO Copy Streams failed", __FILE__, __LINE__);
    }
    fclose (fp);
  } else {
    perror ("fopen");
    fprintf (stderr, "\n%s:%d: fopen failed", __FILE__, __LINE__);
  }
  remove (pathname);
  xfree (pathname);
}

static char *
S_RedirectError ()
{
  char *pathname;

  pathname = IO_MakeUniqueFile ();
  S_saved_stderr = dup (2);
  close (2);
  open (pathname, O_WRONLY, 0600);
  return pathname;
}

static void
S_RestoreError ()
{
  close (2);
  dup (S_saved_stderr);
  close (S_saved_stderr);
  S_saved_stderr = -1;
}

static int
S_Run ()
{
  int rc = 0, i;
  char *pathname, *command;
  char *error_file;

  assert (S_script_pathname != NULL);
  pathname = S_MakeTempfile (S_script_pathname);
  command = LISP_MakeCommand (LISP_PATHNAME, pathname);
  fflush (stdout);
  fflush (stderr);
  error_file = S_RedirectError ();
  i = system (command);
  S_RestoreError ();
  if (i == 0) {
    /* good */
  } else {
    S_PrintError (error_file);
    fprintf (stderr, "\n%s:%d: error: \"%s\"", __FILE__, __LINE__, command);
    rc = 3;
  }
  xfree (command);
  remove (pathname);
  xfree (pathname);
  return rc;
}

static void
S_Uninit ()
{
}

int
main (argc, argv)
     int argc;
     char *argv[];
{
  int rc = 0;

  if (S_Init () == 0) {
    if (S_CommandLine (argc, argv) == 0) {
      if (S_Run () == 0) {
	/* good */
      } else {
	fprintf (stderr, "\n%s:%d: S_Run failed", __FILE__, __LINE__);
	rc = 3;
      }
    } else {
      fprintf (stderr, "\n%s:%d: S_CommandLine failed", __FILE__, __LINE__);
      rc = -3;
    }
    S_Uninit ();
  } else {
    fprintf (stderr, "\n%s:%d: S_Init failed", __FILE__, __LINE__);
    rc = 3;
  }
  return rc == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}

/* --- end of file --- */
